package com.github.polpetta.mezzotre.i18n;

import java.nio.charset.StandardCharsets;
import java.util.Locale;
import java.util.function.Consumer;
import javax.inject.Inject;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.tools.ToolManager;
import org.apache.velocity.util.StringBuilderWriter;

// FIXME doc, tests
public class TemplateContentGenerator {

  private final LocalizedMessageFactory localizedMessageFactory;

  @Inject
  public TemplateContentGenerator(LocalizedMessageFactory localizedMessageFactory) {
    this.localizedMessageFactory = localizedMessageFactory;
  }

  public String mergeTemplate(
      Consumer<VelocityContext> velocityContextConsumer,
      String localeAsString,
      String templateName) {
    final Locale locale = Locale.forLanguageTag(localeAsString);
    final ToolManager toolManager = localizedMessageFactory.createVelocityToolManager(locale);
    final VelocityContext velocityContext = new VelocityContext(toolManager.createContext());

    velocityContextConsumer.accept(velocityContext);

    final StringBuilder content = new StringBuilder();
    final StringBuilderWriter stringBuilderWriter = new StringBuilderWriter(content);

    toolManager
        .getVelocityEngine()
        .mergeTemplate(
            templateName, StandardCharsets.UTF_8.name(), velocityContext, stringBuilderWriter);

    stringBuilderWriter.close();
    return content.toString();
  }

  public String getString(Locale locale, String key) {
    return localizedMessageFactory.createResourceBundle(locale).getString(key);
  }

  public String getString(String localeAsString, String key) {
    return getString(Locale.forLanguageTag(localeAsString), key);
  }
}
